# Globals config keys
TRUSTED_FACES = "trusted_faces"
MOTION_DETECTOR_THRESHOLD = "motion_detector_threshold"
MOTION_DETECTOR_SENSITIVITY = "motion_detector_sensitivity"
TRIGGER_COOLDOWN_TIME = "trigger_cooldown_time"

# Global Default
TRUSTED_FACES_DEFAULT = []
MOTION_DETECTOR_THRESHOLD_DEFAULT = 15  # Magnitude of motion vector
MOTION_DETECTOR_SENSITIVITY_DEFAULT = 25  # Number of motion vectors
TRIGGER_COOLDOWN_TIME_DEFAULT = 5

GLOBALS = {
    TRUSTED_FACES: TRUSTED_FACES_DEFAULT,
    MOTION_DETECTOR_THRESHOLD: MOTION_DETECTOR_THRESHOLD_DEFAULT,
    MOTION_DETECTOR_SENSITIVITY: MOTION_DETECTOR_SENSITIVITY_DEFAULT,
    TRIGGER_COOLDOWN_TIME: TRIGGER_COOLDOWN_TIME_DEFAULT
}
